/*++++++++++++++
.IDENTIFICATION domain.c
.LANGUAGE       C
.AUTHOR         Francois Ochsenbein [CDS]
.ENVIRONMENT    Unix, CDS Catalogues
.KEYWORDS       Table, FITS
.VERSION  1.0   15-May-1991 (anafile)
.VERSION  4.0   28-Feb-1994 New package
.VERSION  4.1   12-Nov-2000: Linux uses "search"
.VERSION  4.2   24-Oct-2005: Bug domain is nul 
			("Sergey E. Koposov" <math@sai.msu.ru>)
.COMMENTS       Find host names, etc
---------------*/

#include <stdio.h>
/*#include <buffer.h>*/
#include <string.h>
#include <ctype.h>

/*==================================================================
		BSD Utilities
 *==================================================================*/
#ifdef __MSDOS__ 	/* For tests */
static int gethost(buf, len)	char *buf; int len ; {
    if (len > 6) strcpy(buf, "PC-DOS");
    return(0);
}
char *getdomain() {
    return("");
}
char *gethostName() {
    return("PC-DOS") ;
}
#else			/* == Standard Unix */
#include <sys/utsname.h>	/* for gethost	*/
char *getdomain()
/*++++++++++++++++
.PURPOSE  Get name of domain (e.g. u-strasbg.fr)
.RETURNS  The domain (no starting dot)
.REMARKS  From /etc/resolv.conf
-----------------*/
{
  static char *domain = (char *)0 ;
  char buf[80], *p ;
  FILE *resolv ;

    if (domain) return(domain) ;

    if (!(resolv = fopen("/etc/resolv.conf", "r")))
	return(domain) ;
    while (fgets(buf, sizeof(buf), resolv)) {
	if (strncmp(buf, "domain", 6) && strncmp(buf, "search", 6)) 
	    continue ;
	p = buf + 6 ; 
	while (isgraph(*p)) p++; 	/* Can be e.g. "domainname" */
	while (isspace(*p)) p++ ;
	domain = strdup(p) ;
	for (p=domain; isgraph(*p); p++)  ;
	*p = 0 ;
	break ;
    }
    fclose(resolv) ;
    return(domain) ;
}

int gethost(host, len)
/*++++++++++++++++
.PURPOSE  Get name of host
.RETURNS  0 (OK) / 1 (truncated) /-1(error)
.REMARKS  BSD compatibility routine. Add the domain name if required.
-----------------*/
  char *host;	/* OUT: Name of host 	*/
  int len;	/* IN: Size of name	*/
{
  struct utsname name; 
  int status, i;
  char *domain, *p ;
    status = uname(&name);
    if (status < 0) return(status);

    for (p=name.nodename; *p && (*p != '.'); p++) ;
    if (*p) domain = "" ;
    else domain = getdomain() ;

    /* Copy the result */
    status = 0 ;
    i = strlen(name.nodename) ; 
    strncpy(host, name.nodename, len); 
    if (i >= len) host[i=len-1]=0, status = 1;

    /* Append the domain */
    if (domain && *domain) {		/* V4.2 */
	p = host + i, len -= i ;
	if (len > 0) *(p++) = '.', len-- ;
	strncpy(p, domain, len) ;
	i = strlen(p) ;
	if (i >= len) p[i=len-1]=0, status = 1;
    }
    return(status);
}

char *gethostName()
/*++++++++++++++++
.PURPOSE  Get Full Name of host
.RETURNS  Name of host
.REMARKS  BSD compatibility routine. Add the domain name if required.
-----------------*/
{
  static char *thename = (char *)0 ;
  char buf[512] ;
    if (thename) return(thename) ;
    if (gethost(buf, sizeof(buf)) == 0) thename = strdup(buf) ;
    return(thename) ;
}
#endif
